/*
 * XMMS Weasel - Plugin for X MultiMedia System (XMMS)
 * 
 * Written by Timo Hoenig and Sebastian Ruml
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
 */

#include "xmms-weasel.h"

/* Global variables */

/*
 * XMMS Weasel config 
 */

Xw_cfg xw_cfg;

Display *dpy;
Window window;

/*
 * ===================================================================================================== 
 */

/*
 * Callback functions 
 */
GeneralPlugin gp = {
	NULL,			/* Handle */
	NULL,			/* Filename */
	-1,			/* XMMS Session */
	"Weasel Plugin",	/* Plugin Description */
	xw_init,		/* XMMS Weasel Init */
	xw_about,		/* XMMS Weasel About */
	xw_config,		/* XMMS Weasel Configuration */
	xw_cleanup		/* XMMS Weasel Cleanup */
};


GeneralPlugin *
get_gplugin_info (void)
{
	gp.description = g_strdup_printf ("Weasel Plugin %s", VERSION);
	return &gp;
};


/*
 * ===================================================================================================== 
 */

void
pointer_timer_hide_stop (void)
{
	if (xw_cfg.pointer_timer.hide_handler_id)
	{
		gtk_timeout_remove (xw_cfg.pointer_timer.hide_handler_id);
		xw_cfg.pointer_timer.hide_handler_id = 0;
	}
}

void
pointer_timer_hide_start ()
{
	pointer_timer_show_stop ();
	xw_cfg.pointer_timer.hide_handler_id =
		gtk_timeout_add (10, (GtkFunction) pointer_timer_hide, NULL);
}

void
pointer_timer_show_stop ()
{
	if (xw_cfg.pointer_timer.show_handler_id)
	{
		gtk_timeout_remove (xw_cfg.pointer_timer.show_handler_id);
		xw_cfg.pointer_timer.show_handler_id = 0;
	}
}

void
pointer_timer_show_start ()
{
	pointer_timer_hide_stop ();
	xw_cfg.pointer_timer.show_handler_id =
		gtk_timeout_add (10, (GtkFunction) pointer_timer_show, NULL);
}

/*
 * ===================================================================================================== 
 */

void
get_window_position (void)
{
	gdk_window_get_position (xw_cfg.main.gdk_window,
				 &xw_cfg.main.x, &xw_cfg.main.y);

	/* We don't have the playlist and equalizer at first, so we break */
	if (!xw_cfg.all_signals_set)
		return;

	gdk_window_get_position (xw_cfg.equalizer.gdk_window,
				 &xw_cfg.equalizer.x, &xw_cfg.equalizer.y);

	gdk_window_get_position (xw_cfg.playlist.gdk_window,
				 &xw_cfg.playlist.x, &xw_cfg.playlist.y);
}

void
get_window_height (void)
{
	gdk_window_get_geometry (xw_cfg.main.gdk_window,
				 NULL, NULL, NULL, &xw_cfg.main.height, NULL);

	gdk_window_get_geometry (xw_cfg.equalizer.gdk_window,
				 NULL, NULL, NULL, &xw_cfg.equalizer.height,
				 NULL);

	gdk_window_get_geometry (xw_cfg.playlist.gdk_window,
				 NULL, NULL, NULL, &xw_cfg.playlist.height,
				 NULL);
}

void
set_window_weasel_speed (void)
{
	/*
	 * Weasel speed depends on the height of a window 
	 */
	get_window_height ();

	switch (xw_cfg.main.height)
	{
	case SHADED:
		if (xw_config_file.speed <= 10)
			xw_cfg.main.weasel_speed = xw_config_file.speed;
		else
			xw_cfg.main.weasel_speed = xw_config_file.speed / 10;
		break;
	case UNSHADED:
		xw_cfg.main.weasel_speed = xw_config_file.speed;
		break;
	default:
		xw_cfg.main.weasel_speed = xw_config_file.speed;
		break;
	}

	switch (xw_cfg.equalizer.height)
	{
	case SHADED:
		if (xw_config_file.speed <= 10)
			xw_cfg.equalizer.weasel_speed = xw_config_file.speed;
		else
			xw_cfg.equalizer.weasel_speed =
				xw_config_file.speed / 10;
		break;
	case UNSHADED:
		xw_cfg.equalizer.weasel_speed = xw_config_file.speed;
		break;
	default:
		xw_cfg.equalizer.weasel_speed = xw_config_file.speed;
		break;
	}

	switch (xw_cfg.playlist.height)
	{
	case SHADED:
		if (xw_config_file.speed <= 10)
			xw_cfg.playlist.weasel_speed = xw_config_file.speed;
		else
			xw_cfg.playlist.weasel_speed =
				xw_config_file.speed / 10;
		break;
	default:
		xw_cfg.playlist.weasel_speed = xw_config_file.speed;
		break;
	}
}

/*
 * Called by pointer_timer_hide() 
 */
void
hide_window (Xw_wininfo * window)
{
	if (window->y > -(window->height - xw_config_file.overlap))
		gdk_window_move (window->gdk_window,
				 window->x, window->y - window->weasel_speed);
	else
	{
		gdk_window_move (window->gdk_window,
				 window->x,
				 -(window->height - xw_config_file.overlap));

		/*
		 * Sometimes my XMMS loses 'always on top'... Don't know why. 
		 */
		hint_set_always(TRUE);
	}

}

/*
 * Called by pointer_timer_show() 
 */
void
show_window (Xw_wininfo * window)
{
	if (window->y + window->weasel_speed < 0)
		gdk_window_move (window->gdk_window,
				 window->x, window->y + window->weasel_speed);
	else if (window->y <= 0)
	{
		gdk_window_move (window->gdk_window, window->x, 0);

	}
	if (window->y >= 0)
		pointer_timer_show_stop ();
}

void
pointer_timer_hide (void)
{
	/* printf("\t\t\t\t\t(hide tick)\n"); */

	if (++xw_cfg.hdelay < xw_config_file.hide_delay)
		return;

	get_window_position ();

	if (xw_cfg.main.y <= 0)
		hide_window (&xw_cfg.main);
	if (xw_cfg.equalizer.y <= 0)
		hide_window (&xw_cfg.equalizer);
	if (xw_cfg.playlist.y <= 0)
		hide_window (&xw_cfg.playlist);

	/*
	 * If all window is completly hidden or somewhere on the screen and
	 * not docked we remove the timer 
	 */
	if (((xw_cfg.main.y == -(xw_cfg.main.height - xw_config_file.overlap))
	     || (xw_cfg.main.y > 0))
	    &&
	    ((xw_cfg.equalizer.y ==
	      -(xw_cfg.equalizer.height - xw_config_file.overlap))
	     || (xw_cfg.equalizer.y > 0))
	    &&
	    ((xw_cfg.playlist.y ==
	      -(xw_cfg.playlist.height - xw_config_file.overlap))
	     || (xw_cfg.playlist.y > 0)))
	{
		pointer_timer_hide_stop ();
	}
}

void
pointer_timer_show (void)
{
	/* printf("\t\t\t\t\t(show tick)\n"); */
	if (++xw_cfg.sdelay < xw_config_file.show_delay)
		return;

	get_window_position ();
	if (xw_cfg.main.y < 0)
		show_window (&xw_cfg.main);
	if (xw_cfg.equalizer.y < 0)
		show_window (&xw_cfg.equalizer);
	if (xw_cfg.playlist.y < 0)
		show_window (&xw_cfg.playlist);

	if ((xw_cfg.main.y >= 0) &&
	    (xw_cfg.equalizer.y >= 0) && (xw_cfg.playlist.y >= 0))
	{
		pointer_timer_show_stop ();
	}
}


/*
 * ===================================================================================================== 
 */

/*
 * Signal handler functions 
 */

void
pointer_leave (GtkWidget * widget, GdkEventMotion * event,
	       gpointer callback_data)
{
#ifdef DEBUG
	printf ("Weasel: pointer_leave() - Pointer left window \"%s\".\n",
		GTK_WINDOW (widget)->title);
#endif
	xw_cfg.sdelay = 0;

	set_window_weasel_speed ();
	pointer_timer_hide_start ();

	/*
	 * Workaround since the playlist and equalizer windows are initialized 
	 * _after_ the plugin 
	 */
	if (!xw_cfg.all_signals_set)
		xw_init_signals ();
}

void
pointer_enter (GtkWidget * widget, GdkEventMotion * event,
	       gpointer callback_data)
{
#ifdef DEBUG
	printf ("Weasel: pointer_enter() - Pointer entered window \"%s\".\n",
		GTK_WINDOW (widget)->title);
#endif

	xw_cfg.hdelay = 0;

	get_window_position ();

	pointer_timer_show_start ();

	/*
	 * Workaround since the playlist and equalizer windows are initialized 
	 * _after_ the plugin 
	 */
	if (!xw_cfg.all_signals_set)
		xw_init_signals ();
}

/*
 * ===================================================================================================== 
 */

void
xw_init_window (Xw_wininfo * window)
{
	window->gdk_window = NULL;
	window->sighandler_pointer_leave = 0;
	window->sighandler_pointer_enter = 0;
	window->weasel_speed = 0;
}

gboolean
xw_init_wm_search (Window window, int level)
{

	int i;
	Window root_win, parent_win;
	unsigned int num_children;
	Window *child_list;
	XClassHint classhint;

	if (!XQueryTree
	    (dpy, window, &root_win, &parent_win, &child_list, &num_children))
		printf ("Can't query window tree.");

	for (i = (int) num_children - 1; i >= 0; i--)
	{
		if (XGetClassHint (dpy, child_list[i], &classhint))
		{
			if (classhint.res_name)
			{
				if (!
				    (strncmp
				     (classhint.res_name, "metacity", 8)))
				{
					/* Metacity found */
					XFree (classhint.res_name);
					if (child_list)
						XFree ((char *) child_list);
					return TRUE;
				}
				XFree (classhint.res_name);
			}
		}

		xw_init_wm_search (child_list[i], level + 1);
	}

	if (child_list)
		XFree ((char *) child_list);

	/* Metacity not found */
	return FALSE;
}

void
xw_init_wm (void)
{
	int screen;

	dpy = XOpenDisplay ("");
	screen = DefaultScreen (dpy);
	window = RootWindow (dpy, screen);

	xw_cfg.metacity_active = xw_init_wm_search (window, 0);
}

void
set_window_dock (GdkWindow * gdk_window)
{
	Atom atom[1];
	int net_wm_window_type;
	int net_wm_window_type_dock;

	window = GDK_WINDOW_XWINDOW (gdk_window);

	net_wm_window_type = XInternAtom (dpy, "_NET_WM_WINDOW_TYPE", False);

	net_wm_window_type_dock = XInternAtom (dpy,
					       "_NET_WM_WINDOW_TYPE_DOCK",
					       False);

	atom[0] = net_wm_window_type_dock;

	/* change property of window */
	gdk_error_trap_push ();

	XChangeProperty (dpy, window, net_wm_window_type,
			 XA_ATOM, 32, PropModeAppend, (guchar *) & atom, 1);

	gdk_error_trap_pop ();
}

void
xw_init_signals (void)
{
	if ((mainwin) && (!xw_cfg.main.sighandler_pointer_leave)
	    && (!xw_cfg.main.sighandler_pointer_enter))
	{
		/* Store widget in config struct */
		xw_cfg.main.gdk_window = mainwin->window;

		/* Metacity Workaround */
		if (xw_cfg.metacity_active)
		{
			/* Pluck widget from the jaws of death (Metacity) */
			gdk_window_set_override_redirect (xw_cfg.main.gdk_window, 1);
		}

		set_window_dock (xw_cfg.main.gdk_window);
		
		/* Attache signal handler */
		xw_cfg.main.sighandler_pointer_leave =
			gtk_signal_connect (GTK_OBJECT (mainwin),
					    "leave_notify_event",
					    GTK_SIGNAL_FUNC (pointer_leave),
					    NULL);
		xw_cfg.main.sighandler_pointer_enter =
			gtk_signal_connect (GTK_OBJECT (mainwin),
					    "enter_notify_event",
					    GTK_SIGNAL_FUNC (pointer_enter),
					    NULL);

		printf ("Weasel: xw_init_signals() - Attached pointer leave/enter signal for mainwin\n");
	}

	if ((playlistwin) && (!xw_cfg.playlist.sighandler_pointer_leave)
	    && (!xw_cfg.playlist.sighandler_pointer_enter))
	{
		/* Store widget in config struct */
		xw_cfg.playlist.gdk_window = playlistwin->window;

		/* Metacity Workaround */
		if (xw_cfg.metacity_active)
		{
			/* Pluck widget from the jaws of death (Metacity) */
			gdk_window_set_override_redirect (xw_cfg.playlist.gdk_window, 1);
		}
		
		set_window_dock (xw_cfg.playlist.gdk_window);
		
		/* Attache signal handler */
		xw_cfg.playlist.sighandler_pointer_leave =
			gtk_signal_connect (GTK_OBJECT (playlistwin),
					    "leave_notify_event",
					    GTK_SIGNAL_FUNC (pointer_leave),
					    NULL);
		xw_cfg.playlist.sighandler_pointer_enter =
			gtk_signal_connect (GTK_OBJECT (playlistwin),
					    "enter_notify_event",
					    GTK_SIGNAL_FUNC (pointer_enter),
					    NULL);

		printf ("Weasel: xw_init_signals() - Attached pointer leave/enter signal for playlistwin\n");
	}

	if ((equalizerwin) && (!xw_cfg.equalizer.sighandler_pointer_leave)
	    && (!xw_cfg.equalizer.sighandler_pointer_enter))
	{
		/* Store widget in config struct */
		xw_cfg.equalizer.gdk_window = equalizerwin->window;

		/* Metacity Workaround */
		if (xw_cfg.metacity_active)
		{
			/* Pluck widget from the jaws of death (Metacity) */
			gdk_window_set_override_redirect (xw_cfg.equalizer.gdk_window, 1);
		}

		set_window_dock (xw_cfg.equalizer.gdk_window);
		
		/* Attache signal handler */
		xw_cfg.equalizer.sighandler_pointer_leave =
			gtk_signal_connect (GTK_OBJECT (equalizerwin),
					    "leave_notify_event",
					    GTK_SIGNAL_FUNC (pointer_leave),
					    NULL);
		xw_cfg.equalizer.sighandler_pointer_enter =
			gtk_signal_connect (GTK_OBJECT (equalizerwin),
					    "enter_notify_event",
					    GTK_SIGNAL_FUNC (pointer_enter),
					    NULL);
		printf ("Weasel: xw_init_signals() - Attached pointer leave/enter" "signal for equalizerwin\n");
	}

	if ((xw_cfg.main.sighandler_pointer_leave)
	    && (xw_cfg.main.sighandler_pointer_enter)
	    && (xw_cfg.playlist.sighandler_pointer_leave)
	    && (xw_cfg.playlist.sighandler_pointer_enter)
	    && (xw_cfg.equalizer.sighandler_pointer_leave)
	    && (xw_cfg.equalizer.sighandler_pointer_enter))
	{
		printf ("Weasel: xw_init_signals() - All signals attached.\n");
		xw_cfg.all_signals_set = TRUE;
	}
}

void
xw_init_pointer_timer (void)
{
	xw_cfg.pointer_timer.hide_handler_id = 0;
	xw_cfg.pointer_timer.show_handler_id = 0;
}

void
xw_init (void)
{
	printf ("Weasel: xw_init()\n");

	xw_init_window (&xw_cfg.main);
	xw_init_window (&xw_cfg.equalizer);
	xw_init_window (&xw_cfg.playlist);

	/* Load configuration */
	xw_load_config ();

	xw_cfg.hdelay = 0;

	xw_init_wm ();

	xw_init_signals ();
	xw_init_pointer_timer ();
};

void
xw_about (void)
{
	static GtkWidget *aboutwin;

	printf ("Weasel: xw_about()\n");

	if (aboutwin)
	{
		gdk_window_raise (aboutwin->window);
		return;
	}

	aboutwin = xmms_show_message ("About Weasel Plugin",
				      "XMMS Weasel Plugin (" VERSION ")\n\n"
				      "Copyright (C) 2004 Timo Hoenig <thoenig@nouse.net>, Sebastian Ruml <sruml@gmx.de>",
				      "Ok", FALSE, NULL, NULL);

	gtk_signal_connect (GTK_OBJECT (aboutwin), "destroy",
			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
			    &aboutwin);
};

void
xw_config (void)
{
	/*
	 * xmms-weaselcfg.c 
	 */
	printf ("Weasel: xw_config()\n");

	if (configwin)
	{
		gdk_window_raise (configwin->window);
		return;
	}

	xw_config_window ();
};

void
xw_cleanup_signals (void)
{
	/*
	 * mainwin signals 
	 */
	if (xw_cfg.main.sighandler_pointer_leave)
	{
		gtk_signal_disconnect (GTK_OBJECT (mainwin),
				       xw_cfg.main.sighandler_pointer_leave);
		xw_cfg.main.sighandler_pointer_leave = 0;
	}
	if (xw_cfg.main.sighandler_pointer_enter)
	{
		gtk_signal_disconnect (GTK_OBJECT (mainwin),
				       xw_cfg.main.sighandler_pointer_enter);
		xw_cfg.main.sighandler_pointer_enter = 0;
	}

	/*
	 * playlistwin signals 
	 */
	if (xw_cfg.playlist.sighandler_pointer_leave)
	{
		gtk_signal_disconnect (GTK_OBJECT (playlistwin),
				       xw_cfg.playlist.
				       sighandler_pointer_leave);
		xw_cfg.playlist.sighandler_pointer_leave = 0;
	}

	if (xw_cfg.playlist.sighandler_pointer_enter)
	{
		gtk_signal_disconnect (GTK_OBJECT (playlistwin),
				       xw_cfg.playlist.
				       sighandler_pointer_enter);
		xw_cfg.playlist.sighandler_pointer_enter = 0;
	}

	/*
	 * equalizerwin signals 
	 */
	if (xw_cfg.equalizer.sighandler_pointer_leave)
	{
		gtk_signal_disconnect (GTK_OBJECT (equalizerwin),
				       xw_cfg.equalizer.
				       sighandler_pointer_leave);
		xw_cfg.playlist.sighandler_pointer_leave = 0;
	}

	if (xw_cfg.equalizer.sighandler_pointer_enter)
	{
		gtk_signal_disconnect (GTK_OBJECT (equalizerwin),
				       xw_cfg.equalizer.
				       sighandler_pointer_enter);
		xw_cfg.equalizer.sighandler_pointer_enter = 0;
	}
}

void
xw_cleanup (void)
{
	printf ("Weasel: xw_cleanup()\n");

	/*
	 * Remove gtk signak handlers 
	 */
	xw_cleanup_signals ();

	/*
	 * Remove any active timer 
	 */
	pointer_timer_show_stop ();
	pointer_timer_hide_stop ();

	/*
	 * Make all XMMS windows visible 
	 */
	gdk_window_move (xw_cfg.main.gdk_window, xw_cfg.main.x, 0);
	gdk_window_move (xw_cfg.equalizer.gdk_window, xw_cfg.equalizer.x, 0);
	gdk_window_move (xw_cfg.playlist.gdk_window, xw_cfg.playlist.x, 0);

};
