/*
 *  XMMS Weasel - Plugin for X MultiMedia System (XMMS)
 *
 *  Written by Timo Hoenig and Sebastian Ruml
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "xmms-weasel.h"

/* Global variables */
static GtkObject *adj_speed;
static GtkObject *adj_overlap;
static GtkObject *adj_show_delay;
static GtkObject *adj_hide_delay;

Xw_config_file xw_config_file;

/*
 * ===================================================================================================== 
 */
/* Callback Functions */
void
xw_chancel_btn (GtkWidget * widget)
{
	gtk_widget_destroy (configwin);
}

void
xw_ok_btn (GtkWidget * widget)
{
	xw_save_config ();
	gtk_widget_destroy (configwin);
}

void
xw_apply_btn (GtkWidget * widget)
{
	xw_save_config ();
}

/*
 * ===================================================================================================== 
 */

void
xw_load_config (void)
{
	ConfigFile *cfile;

	cfile = xmms_cfg_open_default_file ();
	xmms_cfg_read_int (cfile, XW_CONFIG_SECTION, XW_CONFIG_SECTION_SPEED,
			   &xw_config_file.speed);
	xmms_cfg_read_int (cfile, XW_CONFIG_SECTION,
			   XW_CONFIG_SECTION_OVERLAP,
			   &xw_config_file.overlap);
	xmms_cfg_read_int (cfile, XW_CONFIG_SECTION,
			   XW_CONFIG_SECTION_HIDE_DELAY,
			   &xw_config_file.hide_delay);
	xmms_cfg_read_int (cfile, XW_CONFIG_SECTION,
			   XW_CONFIG_SECTION_SHOW_DELAY,
			   &xw_config_file.show_delay);
	xmms_cfg_free (cfile);

	/* Default settings */
	if (!xw_config_file.speed)
	{
		xw_config_file.speed = 15;
		xw_config_file.overlap = 3;
		xw_config_file.hide_delay = 30;
		xw_config_file.show_delay = 30;
	}

	xw_config_file.configured = TRUE;
}

void
xw_save_config (void)
{
	ConfigFile *cfile;

	xw_config_file.speed = GTK_ADJUSTMENT (adj_speed)->value;
	xw_config_file.overlap = GTK_ADJUSTMENT (adj_overlap)->value;
	xw_config_file.hide_delay =
		GTK_ADJUSTMENT (adj_hide_delay)->value / 10;
	xw_config_file.show_delay =
		GTK_ADJUSTMENT (adj_show_delay)->value / 10;

	cfile = xmms_cfg_open_default_file ();

	xmms_cfg_write_int (cfile, XW_CONFIG_SECTION, XW_CONFIG_SECTION_SPEED,
			    xw_config_file.speed);
	xmms_cfg_write_int (cfile, XW_CONFIG_SECTION,
			    XW_CONFIG_SECTION_OVERLAP,
			    xw_config_file.overlap);
	xmms_cfg_write_int (cfile, XW_CONFIG_SECTION,
			    XW_CONFIG_SECTION_HIDE_DELAY,
			    xw_config_file.hide_delay);
	xmms_cfg_write_int (cfile, XW_CONFIG_SECTION,
			    XW_CONFIG_SECTION_SHOW_DELAY,
			    xw_config_file.show_delay);

	xmms_cfg_write_default_file (cfile);
	xmms_cfg_free (cfile);
}

void
xw_config_window (void)
{
	GtkWidget *vbox1;
	GtkWidget *frame1;
	GtkWidget *hbox2;
	GtkWidget *l_sf;
	GtkWidget *hs_speed;
	GtkWidget *frame2;
	GtkWidget *hbox3;
	GtkWidget *l_ov;
	GtkWidget *hs_overlap;
	GtkWidget *frame3;
	GtkWidget *vbox2;
	GtkWidget *hbox4;
	GtkWidget *hbox6;
	GtkWidget *label2;
	GtkWidget *hs_hide_delay;
	GtkWidget *hbox5;
	GtkWidget *label3;
	GtkWidget *hs_show_delay;
	GtkWidget *hbuttonbox1;
	guint btn_Ok_key;
	GtkWidget *btn_Ok;
	guint btn_Apply_key;
	GtkWidget *btn_Apply;
	guint btn_Cancel_key;
	GtkWidget *btn_Cancel;
	GtkAccelGroup *accel_group;

	/* Load configuration */
	if (!xw_config_file.configured)
		xw_load_config ();

	accel_group = gtk_accel_group_new ();

	configwin = gtk_window_new (GTK_WINDOW_DIALOG);
	gtk_object_set_data (GTK_OBJECT (configwin), "configwin", configwin);
	gtk_widget_set_usize (configwin, 440, 300);
	gtk_container_set_border_width (GTK_CONTAINER (configwin), 5);
	gtk_window_set_title (GTK_WINDOW (configwin),
			      "XMMS Weasel Configuartion");
	gtk_window_set_position (GTK_WINDOW (configwin), GTK_WIN_POS_CENTER);
	gtk_window_set_policy (GTK_WINDOW (configwin), FALSE, FALSE, FALSE);

	vbox1 = gtk_vbox_new (FALSE, 5);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "vbox1", vbox1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (configwin), vbox1);

	frame1 = gtk_frame_new ("Weasel Speed");
	gtk_widget_ref (frame1);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "frame1", frame1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (frame1);
	gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);

	hbox2 = gtk_hbox_new (FALSE, 27);
	gtk_widget_ref (hbox2);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hbox2", hbox2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox2);
	gtk_container_add (GTK_CONTAINER (frame1), hbox2);
	gtk_container_set_border_width (GTK_CONTAINER (hbox2), 3);

	l_sf = gtk_label_new ("Speed Factor:");
	gtk_widget_ref (l_sf);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "l_sf", l_sf,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (l_sf);
	gtk_box_pack_start (GTK_BOX (hbox2), l_sf, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (l_sf), GTK_JUSTIFY_RIGHT);

	adj_speed =
		gtk_adjustment_new (xw_config_file.speed, 1.0, 50, 1, 0, 0);
	hs_speed = gtk_hscale_new (GTK_ADJUSTMENT (adj_speed));
	gtk_widget_ref (hs_speed);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hs_speed",
				  hs_speed,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hs_speed);
	gtk_box_pack_start (GTK_BOX (hbox2), hs_speed, TRUE, TRUE, 0);
	gtk_scale_set_value_pos (GTK_SCALE (hs_speed), GTK_POS_RIGHT);
	gtk_scale_set_digits (GTK_SCALE (hs_speed), 0);

	frame2 = gtk_frame_new ("Weasel Overlap");
	gtk_widget_ref (frame2);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "frame2", frame2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (frame2);
	gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);

	hbox3 = gtk_hbox_new (FALSE, 57);
	gtk_widget_ref (hbox3);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hbox3", hbox3,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox3);
	gtk_container_add (GTK_CONTAINER (frame2), hbox3);
	gtk_container_set_border_width (GTK_CONTAINER (hbox3), 3);

	l_ov = gtk_label_new ("Overlap:");
	gtk_widget_ref (l_ov);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "l_ov", l_ov,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (l_ov);
	gtk_box_pack_start (GTK_BOX (hbox3), l_ov, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (l_ov), GTK_JUSTIFY_RIGHT);

	adj_overlap =
		gtk_adjustment_new (xw_config_file.overlap, 1, 15, 1, 0, 0);
	hs_overlap = gtk_hscale_new (GTK_ADJUSTMENT (adj_overlap));
	gtk_widget_ref (hs_overlap);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hs_overlap",
				  hs_overlap,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hs_overlap);
	gtk_box_pack_start (GTK_BOX (hbox3), hs_overlap, TRUE, TRUE, 0);
	gtk_scale_set_value_pos (GTK_SCALE (hs_overlap), GTK_POS_RIGHT);
	gtk_scale_set_digits (GTK_SCALE (hs_overlap), 0);

	frame3 = gtk_frame_new ("Weasel Delay");
	gtk_widget_ref (frame3);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "frame3", frame3,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (frame3);
	gtk_box_pack_start (GTK_BOX (vbox1), frame3, TRUE, TRUE, 0);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox2);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "vbox2", vbox2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox2);
	gtk_container_add (GTK_CONTAINER (frame3), vbox2);

	hbox4 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox4);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hbox4", hbox4,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox4);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox4, TRUE, TRUE, 0);

	hbox6 = gtk_hbox_new (FALSE, 13);
	gtk_widget_ref (hbox6);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hbox6", hbox6,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox6);
	gtk_box_pack_start (GTK_BOX (hbox4), hbox6, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox6), 3);

	label2 = gtk_label_new ("Hide Delay(ms):");
	gtk_widget_ref (label2);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "label2", label2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (hbox6), label2, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_RIGHT);

	adj_hide_delay =
		gtk_adjustment_new (xw_config_file.hide_delay * 10, 0, 1000,
				    1, 0, 0);
	hs_hide_delay = gtk_hscale_new (GTK_ADJUSTMENT (adj_hide_delay));
	gtk_widget_ref (hs_hide_delay);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hs_hide_delay",
				  hs_hide_delay,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hs_hide_delay);
	gtk_box_pack_start (GTK_BOX (hbox6), hs_hide_delay, TRUE, TRUE, 0);
	gtk_scale_set_value_pos (GTK_SCALE (hs_hide_delay), GTK_POS_RIGHT);
	gtk_scale_set_digits (GTK_SCALE (hs_hide_delay), 0);

	hbox5 = gtk_hbox_new (FALSE, 7);
	gtk_widget_ref (hbox5);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hbox5", hbox5,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox5);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox5, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox5), 3);

	label3 = gtk_label_new ("Show Delay(ms):");
	gtk_widget_ref (label3);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "label3", label3,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label3);
	gtk_box_pack_start (GTK_BOX (hbox5), label3, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_RIGHT);

	adj_show_delay =
		gtk_adjustment_new (xw_config_file.show_delay * 10, 0, 1000,
				    1, 0, 0);
	hs_show_delay = gtk_hscale_new (GTK_ADJUSTMENT (adj_show_delay));
	gtk_widget_ref (hs_show_delay);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hs_show_delay",
				  hs_show_delay,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hs_show_delay);
	gtk_box_pack_start (GTK_BOX (hbox5), hs_show_delay, TRUE, TRUE, 0);
	gtk_scale_set_value_pos (GTK_SCALE (hs_show_delay), GTK_POS_RIGHT);
	gtk_scale_set_digits (GTK_SCALE (hs_show_delay), 0);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_ref (hbuttonbox1);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "hbuttonbox1",
				  hbuttonbox1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, FALSE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1),
				   GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox1), 5);
	gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox1), 0,
					   0);

	btn_Ok = gtk_button_new_with_label ("");
	btn_Ok_key =
		gtk_label_parse_uline (GTK_LABEL (GTK_BIN (btn_Ok)->child),
				       "_Ok");
	gtk_widget_add_accelerator (btn_Ok, "clicked", accel_group,
				    btn_Ok_key, GDK_MOD1_MASK,
				    (GtkAccelFlags) 0);
	gtk_widget_ref (btn_Ok);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "btn_Ok", btn_Ok,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (btn_Ok);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), btn_Ok);
	GTK_WIDGET_SET_FLAGS (btn_Ok, GTK_CAN_DEFAULT);

	btn_Apply = gtk_button_new_with_label ("");
	btn_Apply_key =
		gtk_label_parse_uline (GTK_LABEL (GTK_BIN (btn_Apply)->child),
				       "_Apply");
	gtk_widget_add_accelerator (btn_Apply, "clicked", accel_group,
				    btn_Apply_key, GDK_MOD1_MASK,
				    (GtkAccelFlags) 0);
	gtk_widget_ref (btn_Apply);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "btn_Apply",
				  btn_Apply,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (btn_Apply);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), btn_Apply);
	GTK_WIDGET_SET_FLAGS (btn_Apply, GTK_CAN_DEFAULT);

	btn_Cancel = gtk_button_new_with_label ("");
	btn_Cancel_key =
		gtk_label_parse_uline (GTK_LABEL
				       (GTK_BIN (btn_Cancel)->child),
				       "_Cancel");
	gtk_widget_add_accelerator (btn_Cancel, "clicked", accel_group,
				    btn_Cancel_key, GDK_MOD1_MASK,
				    (GtkAccelFlags) 0);
	gtk_widget_ref (btn_Cancel);
	gtk_object_set_data_full (GTK_OBJECT (configwin), "btn_Cancel",
				  btn_Cancel,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (btn_Cancel);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), btn_Cancel);
	GTK_WIDGET_SET_FLAGS (btn_Cancel, GTK_CAN_DEFAULT);

	gtk_window_add_accel_group (GTK_WINDOW (configwin), accel_group);


	gtk_signal_connect (GTK_OBJECT (configwin), "destroy",
			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
			    &configwin);

	gtk_signal_connect_object (GTK_OBJECT (btn_Cancel), "clicked",
				   GTK_SIGNAL_FUNC (xw_chancel_btn), NULL);

	gtk_signal_connect_object (GTK_OBJECT (btn_Ok), "clicked",
				   GTK_SIGNAL_FUNC (xw_ok_btn), NULL);

	gtk_signal_connect_object (GTK_OBJECT (btn_Apply), "clicked",
				   GTK_SIGNAL_FUNC (xw_apply_btn), NULL);


	gtk_widget_show_all (configwin);
}
